//!
//!

#[macro_use]
extern crate diesel;
#[macro_use]
extern crate diesel_migrations;

use diesel::{
  r2d2::{ConnectionManager, Pool},
  PgConnection,
};
use std::env;

mod compositions;
mod repositories;
mod services;

diesel_migrations::embed_migrations!();

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
  let url_server_broker = match env::var_os("URL_BROKER") {
    Some(v) => v.into_string().unwrap(),
    None => "".to_string(),
  };
  let queue_server_broker = match env::var_os("QUEUE_BROKER") {
    Some(v) => v.into_string().unwrap(),
    None => "".to_string(),
  };

  let url_db = match env::var_os("URL_DATABASE") {
    Some(v) => v.into_string().unwrap(),
    None => "".to_string(),
  };

  let captcha_salt = match env::var_os("CAPTCHA_SALT") {
    Some(v) => v.into_string().unwrap(),
    None => "".to_string(),
  };
  let password_salt = match env::var_os("PASSWORD_SALT") {
    Some(v) => v.into_string().unwrap(),
    None => "".to_string(),
  };
  let jwt_secret = match env::var_os("JWT_SECRET") {
    Some(v) => v.into_string().unwrap(),
    None => "".to_string(),
  };

  let manager = ConnectionManager::<PgConnection>::new(url_db);
  let pool = Pool::builder()
    .max_size(4)
    .test_on_check_out(true)
    .build(manager)?;
  let conn = pool.get()?;

  embedded_migrations::run(&conn)?;

  let server_broker_opts = nats::asynk::Options::new();
  let connection_server_broker = server_broker_opts.connect(url_server_broker).await?;

  let identity_service = services::identity::service::Service::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
    captcha_salt.clone(),
    password_salt.clone(),
    jwt_secret.clone(),
  );

  println!("Server service [beteran-server-service] is started");

  futures::try_join!(identity_service.subscribe(),)?;

  Ok(())
}
